#include <arpa/inet.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>

#include <sys/socket.h>
#include <sys/syscall.h>
#include <unistd.h>

#include <stdlib.h>
#include <stdio.h>

#include <errno.h>

static inline void printsockinfo(int sfd)
{
	int myerrno;
	int prio;
	int dscp;
	int ka;
	int kidle;
	socklen_t optlen;

	optlen = sizeof(int);
	if (getsockopt(sfd, SOL_SOCKET, SO_PRIORITY, &prio, &optlen))
	{
		myerrno = errno;
		perror("SOL_SOCKET getsockopt()");
		_exit(myerrno);
	}

	optlen = sizeof(int);
	if (getsockopt(sfd, IPPROTO_IP, IP_TOS, &dscp, &optlen))
	{
		myerrno = errno;
		perror("IPPROTO_IP getsockopt()");
		_exit(myerrno);
	}

	optlen = sizeof(int);
	if (getsockopt(sfd, SOL_SOCKET, SO_KEEPALIVE, &ka, &optlen))
	{
		myerrno = errno;
		perror("SOL_SOCKET getsockopt()");
		_exit(myerrno);
	}

	optlen = sizeof(int);
	if (getsockopt(sfd, IPPROTO_TCP, TCP_KEEPIDLE, &kidle, &optlen))
	{
		myerrno = errno;
		perror("IPPROTO_TCP getsockopt()");
		_exit(myerrno);
	}

	printf("priority = %i, dscp = %02x, keepalive = %i, kidle = %i\n", prio, dscp, ka, kidle);
}

int main(int argc, char *argv[])
{
	pid_t pid;
	int fd;
	char *endptr;
	int pidfd;
	int sock;
	int myerrno;

	if (argc != 3)
	{
		printf("Usage:\n\t%s PID FILE_DESCRIPTOR_NUM\n\nDisplays the socket info (including dscp) for a socket in a process with a given\nPID and FILE_DESCRIPTOR_NUM.\n", argc ? argv[0] : "UNKNOWN");
		return EINVAL;
	}

	pid = strtoul(argv[1], &endptr, 0);
	if (*endptr)
	{
		myerrno = errno;
		puts("unable to parse PID");
		if (!myerrno)
		{
			return EINVAL;
		}
		return myerrno;
	}

	fd = strtoul(argv[2], &endptr, 0);
	if (*endptr)
	{
		myerrno = errno;
		puts("unable to parse FILE_DESCRIPTOR_NUM");
		if (!myerrno)
		{
			return EINVAL;
		}
		return myerrno;
	}

	pidfd = syscall(SYS_pidfd_open, pid, 0);
	if (pidfd == -1)
	{
		myerrno = errno;
		perror("pidfd_open()");
		return myerrno;
	}

	sock = syscall(SYS_pidfd_getfd, pidfd, fd, 0);
	if (sock == -1)
	{
		myerrno = errno;
		close(pidfd);
		perror("pidfd_getfd()");
		return myerrno;
	}

	printsockinfo(sock);

	//I hope this doesn't fucking close the fd
	//for everybody!
	close(sock);
	close(pidfd);
	

	return 0;
}
